﻿using C1.Web.Mvc;
using C1.Web.Mvc.Serialization;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using FlexGrid.Models;
using System.Data.Entity.Validation;
using System.Data.Entity;
using System.Data;

namespace FlexGrid.Controllers
{
    public class FlexGridController : Controller
    {
        // Define dataSource for FlexGrid
        private C1NWindEntities db = new C1NWindEntities();
        // GET: FlexGrid
        public ActionResult Index()
        {            
            return View(db.Products.ToList());
        }
        //Instantiate a JSON request
        public ActionResult GridReadProduct([C1JsonRequest] CollectionViewRequest<Product> requestData)
        {
            return this.C1Json(CollectionViewHelper.Read(requestData, db.Products));
        }

        public ActionResult GridCreateProduct([C1JsonRequest]CollectionViewEditRequest<Product> requestData)
        {
            var Product = requestData.OperatingItems.First();
            if (Product.ProductName == null)
            {
                Product.ProductName = "";
            }
            return Create(requestData, db.Products);
        }
        /// <summary>
        /// Create a new item to the DataBase
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="requestData">CollectionViewEditRequest Data</param>
        /// <param name="data">DataBase Object</param>
        /// <returns></returns>
        private ActionResult Create<T>(CollectionViewEditRequest<T> requestData, DbSet<T> data) where T : class
        {
            return this.C1Json(CollectionViewHelper.Edit<T>(requestData, item =>
            {
                string error = string.Empty;
                bool success = true;
                try
                {
                    data.Add(item);
                    db.SaveChanges();
                }
                catch (DbEntityValidationException e)
                {
                    error = string.Join(",", e.EntityValidationErrors.Select(result =>
                    {
                        return string.Join(",", result.ValidationErrors.Select(err => err.ErrorMessage));
                    }));
                    success = false;
                }
                catch (Exception e)
                {
                    error = e.Message;
                    success = false;
                }
                return new CollectionViewItemResult<T>
                {
                    Error = error,
                    Success = success && ModelState.IsValid,
                    Data = item
                };
            }, () => data.ToList<T>()));
        }
        /// <summary>
        /// Update Edited item to the DataBase
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="requestData">CollectionViewEditRequest Data</param>
        /// <param name="data">DataBase Object</param>
        /// <returns></returns>
        private ActionResult Update<T>(CollectionViewEditRequest<T> requestData, DbSet<T> data) where T : class
        {
            return this.C1Json(CollectionViewHelper.Edit<T>(requestData, item =>
            {
                string error = string.Empty;
                bool success = true;
                try
                {
                    db.Entry(item as object).State = EntityState.Modified;
                    db.SaveChanges();
                }
                catch (DbEntityValidationException e)
                {
                    error = string.Join(",", e.EntityValidationErrors.Select(result =>
                    {
                        return string.Join(",", result.ValidationErrors.Select(err => err.ErrorMessage));
                    }));
                    success = false;
                }
                catch (Exception e)
                {
                    error = e.Message;
                    success = false;
                }
                return new CollectionViewItemResult<T>
                {
                    Error = error,
                    Success = success && ModelState.IsValid,
                    Data = item
                };
            }, () => data.ToList<T>()));
        }

        //Instantiate a JSON request
        public ActionResult GridUpdateProduct([C1JsonRequest]CollectionViewEditRequest<Product> requestData)
        {
            return Update(requestData, db.Products);
        }

        // Batch update

        public ActionResult GridBatchEdit([C1JsonRequest]CollectionViewBatchEditRequest<Product> requestData)
        {
            return this.C1Json(CollectionViewHelper.BatchEdit(requestData, batchData =>
            {
                var itemresults = new List<CollectionViewItemResult<Product>>();
                string error = string.Empty;
                bool success = true;
                try
                {
                    if (batchData.ItemsCreated != null)
                    {
                        batchData.ItemsCreated.ToList().ForEach(st =>
                        {
                            db.Products.Add(st);
                            itemresults.Add(new CollectionViewItemResult<Product>
                            {
                                Error = "",
                                Success = ModelState.IsValid,
                                Data = st
                            });
                        });
                    }
                    if (batchData.ItemsDeleted != null)
                    {
                        batchData.ItemsDeleted.ToList().ForEach(product =>
                        {
                            var fProduct = db.Products.Find(product.ProductID);
                            db.Products.Remove(fProduct);
                            itemresults.Add(new CollectionViewItemResult<Product>
                            {
                                Error = "",
                                Success = ModelState.IsValid,
                                Data = product
                            });
                        });
                    }
                    if (batchData.ItemsUpdated != null)
                    {
                        batchData.ItemsUpdated.ToList().ForEach(product =>
                        {
                            db.Entry(product).State = EntityState.Modified;
                            itemresults.Add(new CollectionViewItemResult<Product>
                            {
                                Error = "",
                                Success = ModelState.IsValid,
                                Data = product
                            });
                        });
                    }
                    db.SaveChanges();
                }
                catch (DbEntityValidationException e)
                {
                    error = string.Join(",", e.EntityValidationErrors.SelectMany(i => i.ValidationErrors).Select(i => i.ErrorMessage));
                    success = false;
                }
                catch (Exception e)
                {
                    error = e.Message;
                    success = false;
                }

                return new CollectionViewResponse<Product>
                {
                    Error = error,
                    Success = success,
                    OperatedItemResults = itemresults
                };
            }, () => db.Products.ToList()));
        }
        [HttpGet]

        public ActionResult ProductEdit(Product model)
        {
            if (ModelState.IsValid) {
                return PartialView("<h1> Suceed</h1>");
            }
            return PartialView("_ProductEdit", model);
        }
    }

}